import * as minecraft from "@minecraft/server"

const dowsingRods = {
  "dowsing:acacia_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "dripstone_block", name: "Dripstone Caves"},
      {block: "moss_block", name: "Lush Caves"},
      {block: "clay", name: "Lush Caves"},
      {block: "sculk", name: "Deep Dark"},
    ],
    distance: 64
  },
  "dowsing:acacia_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "dripstone_block", name: "Dripstone Caves"},
      {block: "moss_block", name: "Lush Caves"},
      {block: "clay", name: "Lush Caves"},
      {block: "sculk", name: "Deep Dark"},
    ],
    distance: 128
  },
  "dowsing:birch_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "*ore"},
      {block: "#dowsing:overworld_ore"}
    ],
    distance: 64
  },
  "dowsing:birch_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "*ore"},
      {block: "#dowsing:overworld_ore"}
    ],
    distance: 128
  },
  "dowsing:crimson_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "lava", name: "Lava"},
      {block: "flowing_lava", name: "Lava"}
    ],
    distance: 64
  },
  "dowsing:crimson_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "lava", name: "Lava"},
      {block: "flowing_lava", name: "Lava"}
    ],
    distance: 128
  },
  "dowsing:dark_oak_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "calcite", name: "Geode"},
      {block: "smooth_basalt", name: "Geode"},
      {block: "#dowsing:geode_wall", name: "Geode"}
    ],
    distance: 64
  },
  "dowsing:dark_oak_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "calcite", name: "Geode"},
      {block: "smooth_basalt", name: "Geode"},
      {block: "#dowsing:geode_wall", name: "Geode"}
    ],
    distance: 128
  },
  "dowsing:jungle_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "rail", name: "Structure"},
      {block: "cobblestone", name: "Structure"},
      {block: "mossy_cobblestone", name: "Structure"},
      {block: "planks", name: "Structure"},
      {block: "web", name: "Structure"},
      {block: "stonebrick", name: "Structure"}
    ],
    distance: 64
  },
  "dowsing:jungle_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "rail", name: "Structure"},
      {block: "cobblestone", name: "Structure"},
      {block: "mossy_cobblestone", name: "Structure"},
      {block: "planks", name: "Structure"},
      {block: "web", name: "Structure"},
      {block: "stonebrick", name: "Structure"}
    ],
    distance: 128
  },
  "dowsing:oak_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "#water", name: "Water"}
    ],
    distance: 64
  },
  "dowsing:oak_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "#water", name: "Water"}
    ],
    distance: 128
  },
  "dowsing:spruce_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "air", name: "Cave"}
    ],
    distance: 64
  },
  "dowsing:spruce_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "air", name: "Cave"}
    ],
    distance: 128
  },
  "dowsing:warped_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "*ore"},
      {block: "ancient_debris"},
      {block: "#dowsing:nether_ore"}
    ],
    distance: 64
  },
  "dowsing:warped_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "*ore"},
      {block: "ancient_debris"},
      {block: "#dowsing:nether_ore"}
    ],
    distance: 128
  },
  "dowsing:cherry_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "stone", data: "stone_type", value: "granite", name: "Granite"},
      {block: "stone", data: "stone_type", value: "diorite", name: "Diorite"},
      {block: "stone", data: "stone_type", value: "andesite", name: "Andesite"},
      {block: "tuff"},
      {block: "blackstone"},
      {block: "#dowsing:decorative_stone"}
    ],
    distance: 64
  },
  "dowsing:cherry_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "stone", data: "stone_type", value: "granite", name: "Granite"},
      {block: "stone", data: "stone_type", value: "diorite", name: "Diorite"},
      {block: "stone", data: "stone_type", value: "andesite", name: "Andesite"},
      {block: "tuff"},
      {block: "blackstone"},
      {block: "#dowsing:decorative_stone"}
    ],
    distance: 128
  },
  "dowsing:mangrove_dowsing_rod": {
    cooldown: 60,
    search: [
      {block: "minecraft:*", invert: true}
    ],
    distance: 64
  },
  "dowsing:mangrove_dowsing_rod_upgraded": {
    cooldown: 20,
    search: [
      {block: "minecraft:*", invert: true}
    ],
    distance: 128
  }
};

function constructMessage(block, search){
  if(search.name){
    return `{"text":"${search.name}"}`;
  }else{
    return `{"translate":"tile.${block.typeId.startsWith("minecraft:") ? block.typeId.slice(10) : block.typeId}.name"}`;
  }
}

function checkBlock(block, rod, found){
  rod.search.forEach(search => {
    var condition = false;

    if(search.block.startsWith("#")){
      condition = block.hasTag(search.block.slice(1));
    }else if(search.block.includes("*")){
      if(search.block.startsWith("*") && search.block.endsWith("*")){
        condition = block.typeId.includes(search.block.slice(1, search.block.length - 2));
      }else if(search.block.startsWith("*")){
        condition = block.typeId.endsWith(search.block.slice(1));
      }else if(search.block.endsWith("*")){
        condition = block.typeId.startsWith(search.block.slice(0, search.block.length - 2));
      }
    }else{
      condition = block.typeId == search.block || block.typeId == "minecraft:" + search.block;
    }

    if((!search.invert && !condition) || (search.invert && condition)){
      return;
    }

    if(search.data){
      var state;
      if(block.permutation.getState){
        state = block.permutation.getState(search.data)
      }else{
        state = block.permutation.getProperty(search.data);
      }

      if(!state || state !== search.value){
        return;
      }
    }

    found.push(constructMessage(block, search));
  });
}

const events = minecraft.world.afterEvents || minecraft.world.events;

events.itemUse.subscribe(e => {
  const player = e.source;
  const inv = player.getComponent("inventory").container;
  const item = inv.getItem(player.selectedSlot);

  if(!dowsingRods[item.typeId]){
    return;
  }

  const cooldown = player.getItemCooldown(item.getComponent("minecraft:cooldown").cooldownCategory);
  const rod = dowsingRods[item.typeId];

  if(cooldown > 0 && !(cooldown > rod.cooldown - 2)){
    return;
  }

  const dim = player.dimension;

  var found = [];

  for(var i = player.location.y - 1;(i > (dim.id === "nether" || dim.id === "the_end" ? 1 : -63) && (i > player.location.y - rod.distance));i--){
    try {
      checkBlock(dim.getBlock({
        x: player.location.x,
        y: i,
        z: player.location.z
      }), rod, found);
      checkBlock(dim.getBlock({
        x: player.location.x + 1,
        y: i,
        z: player.location.z
      }), rod, found);
      checkBlock(dim.getBlock({
        x: player.location.x - 1,
        y: i,
        z: player.location.z
      }), rod, found);
      checkBlock(dim.getBlock({
        x: player.location.x,
        y: i,
        z: player.location.z + 1
      }), rod, found);
      checkBlock(dim.getBlock({
        x: player.location.x,
        y: i,
        z: player.location.z - 1
      }), rod, found);
    } catch {
      break;
    }
  }

  if(found.length){
    found = [...new Set(found)];
    player.runCommand(`tellraw @s {"rawtext":[{"text":"§7Found "},${found.join(`,{"text":", "},`)}]}`);
  }else{
    player.runCommand(`tellraw @s {"rawtext":[{"text":"§7Nothing found"}]}`);
  }
});